@@ -7,7 +7,7 @@ use Types::Standard qw(Str Bool Ref InstanceOf);
 use Types::URI -all;
 
 our $AUTHORITY = 'cpan:KJETILK';
-our $VERSION   = '0.01';
+our $VERSION   = '0.02';
 
 =pod
 
@@ -24,7 +24,7 @@ Compose it into a class, e.g.
   package LWP::UserAgent::MyCacher;
   use Moo;
   extends 'LWP::UserAgent';
-  with 'LWP::UserAgent::Role::CHICaching';
+  with 'LWP::UserAgent::Role::CHICaching', 'LWP::UserAgent::Role::CHICaching::SimpleKeyGen';
 
 
 =head1 DESCRIPTION
@@ -46,11 +46,7 @@ class you can use.
 
 Used to set the C<CHI> object to be used as cache in the constructor.
 
-=item C<< key >>, C<< clear_key >>
 
-The key to use for a response. Defaults to the canonical URI of the
-request. May make sense to set differently in the future, but should
-probably be left alone for now.
 
 =item C<< request_uri >>
 
@@ -74,6 +70,27 @@ heuristic lifetime. By default, heuristic freshness lifetime is off,
 only standards-compliant freshness lifetime (i.e. based on the
 C<Cache-Control> and C<Expires> headers) are used.
 
+=back
+
+=head2 Implemented elsewhere
+
+The following are required by this role, but implemented
+elsewhere. See L<LWP::UserAgent::Role::CHICaching::SimpleKeyGen> for
+further explanations.
+
+=over
+
+=item C<< key >>, C<< clear_key >>
+
+The key to use for a response.
+
+=item C<< cache_vary($response) >>
+
+A method that returns true if the response may be cached even if it
+contains a C<Vary> header, false otherwise. The L<HTTP::Response>
+object will be passed to it as a parameter.
+
+
 =back
 
 =cut
@@ -84,14 +101,9 @@ has cache => (
 				  required => 1,
 				 );
 
-has key => (
-				is => 'rw',
-				isa => Str,
-				lazy => 1,
-				clearer => 1,
-				builder => sub { shift->request_uri->canonical->as_string }
-			  );
 
+requires 'key';
+requires 'cache_vary';
 
 has request_uri => (
 						  is =>'rw',
@@ -123,11 +135,10 @@ around request => sub {
 	my $cached = $self->cache->get($self->key); # CHI will take care of expiration
 
 	my $expires_in = 0;
-	if (defined($cached) && (!defined ($cached->header('Vary')))) {
+	if (defined($cached)) {
 		######## Here, we decide whether to reuse a cached response.
 		######## The standard describing this is:
 		######## http://tools.ietf.org/html/rfc7234#section-4
-		# TODO: Vary is complex, we can't support that for now
 		$cached->header('Age' => $cached->current_age);
 		return $cached;
 	} else {
@@ -146,6 +157,12 @@ around request => sub {
 
 		## o  the response status code is understood by the cache, and
 		if ($res->is_success) { # TODO: Cache only successful responses for now
+
+			# First, we deal superficially with the Vary header, for the
+			# full complexity see
+			# http://tools.ietf.org/html/rfc7234#section-4.1
+			return $res unless ($self->cache_vary($res));
+
 			my $cc = join('|',$res->header('Cache-Control')); # Since we only do string matching, this should be ok
 			if (defined($cc)) {
 				## o  the "no-store" cache directive (see Section 5.2) does not appear
@@ -207,10 +224,9 @@ __END__
 
 Will only cache C<GET> requests, and only successful responses.
 
-It does not consider the C<Vary> field.
-
 The module does not validate and does not serve stale responses, even
-when it would be allowed to do so.
+when it would be allowed to do so. It nevertheless does most of
+RFC7234.
 
 =head1 BUGS
 
